@@ -7,6 +7,7 @@ import cgi
 import time
 import re
 from operator import  itemgetter
+from anki.lang import ngettext
 
 from aqt.qt import *
 import anki
@@ -234,6 +235,8 @@ class DataModel(QAbstractTableModel):
             return str(c.reps)
         elif type == "cardLapses":
             return str(c.lapses)
+        elif type == "noteTags":
+            return " ".join(c.note().tags)
         elif type == "note":
             return c.model()['name']
         elif type == "cardIvl":
@@ -487,6 +490,7 @@ class Browser(QMainWindow):
             ('cardEase', _("Ease")),
             ('cardReps', _("Reviews")),
             ('cardLapses', _("Lapses")),
+            ('noteTags', _("Tags")),
             ('note', _("Note")),
         ]
         self.columns.sort(key=itemgetter(1))
@@ -631,7 +635,7 @@ class Browser(QMainWindow):
 
     def onSortChanged(self, idx, ord):
         type = self.model.activeCols[idx]
-        noSort = ("question", "answer", "template", "deck", "note")
+        noSort = ("question", "answer", "template", "deck", "note", "noteTags")
         if type in noSort:
             if type == "template":
                 showInfo(_("""\
@@ -691,12 +695,19 @@ by clicking on one on the left."""))
             if len(self.model.activeCols) < 2:
                 return showInfo(_("You must have at least one column."))
             self.model.activeCols.remove(type)
+            adding=False
         else:
             self.model.activeCols.append(type)
+            adding=True
         # sorted field may have been hidden
         self.setSortIndicator()
         self.setColumnSizes()
         self.model.endReset()
+        # if we added a column, scroll to it
+        if adding:
+            row = self.currentRow()
+            idx = self.model.index(row, len(self.model.activeCols) - 1)
+            self.form.tableView.scrollTo(idx)
 
     def setColumnSizes(self):
         hh = self.form.tableView.horizontalHeader()
@@ -1321,6 +1332,7 @@ update cards set usn=?, mod=?, did=? where id in """ + scids,
         restoreGeom(d, "findDupes")
         fields = sorted(anki.find.fieldNames(self.col, downcase=False))
         frm.fields.addItems(fields)
+        self._dupesButton = None
         # links
         frm.webView.page().setLinkDelegationPolicy(
             QWebPage.DelegateAllLinks)
@@ -1332,15 +1344,19 @@ update cards set usn=?, mod=?, did=? where id in """ + scids,
         self.connect(d, SIGNAL("finished(int)"), onFin)
         def onClick():
             field = fields[frm.fields.currentIndex()]
-            self.duplicatesReport(frm.webView, field, frm.search.text())
+            self.duplicatesReport(frm.webView, field, frm.search.text(), frm)
         search = frm.buttonBox.addButton(
             _("Search"), QDialogButtonBox.ActionRole)
         self.connect(search, SIGNAL("clicked()"), onClick)
         d.show()
 
-    def duplicatesReport(self, web, fname, search):
+    def duplicatesReport(self, web, fname, search, frm):
         self.mw.progress.start()
         res = self.mw.col.findDupes(fname, search)
+        if not self._dupesButton:
+            self._dupesButton = b = frm.buttonBox.addButton(
+                _("Tag Duplicates"), QDialogButtonBox.ActionRole)
+            self.connect(b, SIGNAL("clicked()"), lambda: self._onTagDupes(res))
         t = "<html><body>"
         groups = len(res)
         notes = sum(len(r[1]) for r in res)
@@ -1358,6 +1374,20 @@ update cards set usn=?, mod=?, did=? where id in """ + scids,
         web.setHtml(t)
         self.mw.progress.finish()
 
+    def _onTagDupes(self, res):
+        if not res:
+            return
+        self.model.beginReset()
+        self.mw.checkpoint(_("Tag Duplicates"))
+        nids = set()
+        for s, nidlist in res:
+            nids.update(nidlist)
+        self.col.tags.bulkAdd(nids, _("duplicate"))
+        self.mw.progress.finish()
+        self.model.endReset()
+        self.mw.requireReset()
+        tooltip(_("Notes tagged."))
+
     def dupeLinkClicked(self, link):
         self.form.searchEdit.lineEdit().setText(link.toString())
         self.onSearch()
