@@ -1,13 +1,22 @@
+# coding=utf-8
 # Copyright: Damien Elmes <anki@ichi2.net>
 # License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html
 
-import os, re, traceback, zipfile, json
+import os
+import re
+import traceback
+import zipfile
+import json
+
 from aqt.qt import *
 import anki.importing as importing
 from aqt.utils import getOnlyText, getFile, showText, showWarning, openHelp,\
     askUser, tooltip
 from anki.hooks import addHook, remHook
-import aqt.forms, aqt.modelchooser, aqt.deckchooser
+import aqt.forms
+import aqt.modelchooser
+import aqt.deckchooser
+
 
 class ChangeMap(QDialog):
     def __init__(self, mw, model, current):
@@ -145,6 +154,7 @@ you can enter it here. Use \\t to represent tab."""),
         if did != self.importer.model['did']:
             self.importer.model['did'] = did
             self.mw.col.models.save(self.importer.model)
+        self.mw.col.decks.select(did)
         self.mw.progress.start(immediate=True)
         self.mw.checkpoint(_("Import"))
         try:
@@ -154,9 +164,11 @@ you can enter it here. Use \\t to represent tab."""),
             return
         except Exception, e:
             msg = _("Import failed.\n")
-            err = unicode(e)
+            err = repr(str(e))
             if "1-character string" in err:
                 msg += err
+            elif "invalidTempFolder" in err:
+                msg += self.mw.errorHandler.tempFolderMsg()
             else:
                 msg += unicode(traceback.format_exc(), "ascii", "replace")
             showText(msg)
@@ -282,7 +294,7 @@ def importFile(mw, file):
             showUnicodeWarning()
             return
         except Exception, e:
-            msg = unicode(e)
+            msg = repr(str(e))
             if msg == "unknownFormat":
                 if file.endswith(".anki2"):
                     showWarning(_("""\
@@ -299,8 +311,15 @@ backup, please see the 'Backups' section of the user manual."""))
             mw.progress.finish()
         diag = ImportDialog(mw, importer)
     else:
-        # if it's an apkg, we need to ask whether to import/replace
+        # if it's an apkg/zip, first test it's a valid file
         if importer.__class__.__name__ == "AnkiPackageImporter":
+            z = zipfile.ZipFile(importer.file)
+            try:
+                z.getinfo("collection.anki2")
+            except:
+                showWarning(_("The provided file is not a valid .apkg file."))
+                return
+            # we need to ask whether to import/replace
             if not setupApkgImport(mw, importer):
                 return
         mw.progress.start(immediate=True)
@@ -314,11 +333,12 @@ failed. Please try again, and if the problem persists, please try again \
 with a different browser.""")
             showWarning(msg)
         except Exception, e:
-            if "invalidFile" in unicode(e):
+            err = repr(str(e))
+            if "invalidFile" in err:
                 msg = _("""\
 Invalid file. Please restore from backup.""")
                 showWarning(msg)
-            elif "readonly" in unicode(e):
+            elif "readonly" in err:
                 showWarning(_("""\
 Unable to import from a read-only file."""))
             else:
