@@ -2,19 +2,28 @@
 # -*- coding: utf-8 -*-
 # License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html
 
-import os, sys, re, traceback, signal
+import os
+import sys
+import re
+import traceback
+import signal
 import  zipfile
+
 from send2trash import send2trash
 from aqt.qt import *
-
 from anki import Collection
 from anki.utils import  isWin, isMac, intTime, splitFields, ids2str
-from anki.hooks import runHook, addHook
 
-import aqt, aqt.progress, aqt.webview, aqt.toolbar, aqt.stats
+from anki.hooks import runHook, addHook
+import aqt
+import aqt.progress
+import aqt.webview
+import aqt.toolbar
+import aqt.stats
 from aqt.utils import  restoreGeom, showInfo, showWarning,\
     restoreState, getOnlyText, askUser, applyStyles, showText, tooltip, \
     openHelp, openLink, checkInvalidFilename
+import anki.db
 
 class AnkiQt(QMainWindow):
     def __init__(self, app, profileManager, args):
@@ -54,7 +63,7 @@ class AnkiQt(QMainWindow):
                     "syncing and add-on loading."))
         # were we given a file to import?
         if args and args[0]:
-            self.onAppMsg(unicode(args[0], "utf8", "ignore"))
+            self.onAppMsg(unicode(args[0], sys.getfilesystemencoding(), "ignore"))
         # Load profile in a timer so we can let the window finish init and not
         # close on profile load error.
         self.progress.timer(10, self.setupProfile, False)
@@ -159,7 +168,6 @@ class AnkiQt(QMainWindow):
         return True
 
     def profileNameOk(self, str):
-        from anki.utils import invalidFilename, invalidFilenameChars
         return not checkInvalidFilename(str)
 
     def onAddProfile(self):
@@ -220,15 +228,20 @@ Are you sure?""")):
         self.activateWindow()
         self.raise_()
         # maybe sync (will load DB)
-        self.onSync(auto=True)
+        if self.pendingImport and os.path.basename(
+                self.pendingImport).startswith("backup-"):
+            # skip sync when importing a backup
+            self.loadCollection()
+        else:
+            self.onSync(auto=True)
         # import pending?
         if self.pendingImport:
             if self.pm.profile['key']:
                 showInfo(_("""\
 To import into a password protected profile, please open the profile before attempting to import."""))
             else:
-                import aqt.importing
-                aqt.importing.importFile(self, self.pendingImport)
+                self.handleImport(self.pendingImport)
+
             self.pendingImport = None
         runHook("profileLoaded")
 
@@ -255,13 +268,23 @@ To import into a password protected profile, please open the profile before atte
     def loadCollection(self):
         self.hideSchemaMsg = True
         try:
-            self.col = Collection(self.pm.collectionPath())
-        except:
+            self.col = Collection(self.pm.collectionPath(), log=True)
+        except anki.db.Error:
             # move back to profile manager
             showWarning("""\
 Your collection is corrupt. Please see the manual for \
 how to restore from a backup.""")
-            return self.unloadProfile()
+            self.unloadProfile()
+            raise
+        except Exception, e:
+            # the custom exception handler won't catch this if we immediately
+            # unload, so we have to manually handle it
+            if "invalidTempFolder" in repr(str(e)):
+                showWarning(self.errorHandler.tempFolderMsg())
+                self.unloadProfile()
+                return
+            self.unloadProfile()
+            raise
         self.hideSchemaMsg = False
         self.progress.setupDB(self.col.db)
         self.maybeEnableUndo()
@@ -280,7 +303,10 @@ how to restore from a backup.""")
                 return
             self.maybeOptimize()
             self.progress.start(immediate=True)
-            corrupt = self.col.db.scalar("pragma integrity_check") != "ok"
+            if os.getenv("ANKIDEV", 0):
+                corrupt = False
+            else:
+                corrupt = self.col.db.scalar("pragma integrity_check") != "ok"
             if corrupt:
                 showWarning(_("Your collection file appears to be corrupt. \
 This can happen when the file is copied or moved while Anki is open, or \
@@ -299,7 +325,7 @@ the manual for information on how to restore from an automatic backup."))
 
     def backup(self):
         nbacks = self.pm.profile['numBackups']
-        if not nbacks:
+        if not nbacks or os.getenv("ANKIDEV", 0):
             return
         dir = self.pm.backupFolder()
         path = self.pm.collectionPath()
@@ -328,7 +354,7 @@ the manual for information on how to restore from an automatic backup."))
             delete = len(backups) + 1 - nbacks
             delete = backups[:delete]
             for file in delete:
-                send2trash(os.path.join(dir, file[1]))
+                os.unlink(os.path.join(dir, file[1]))
 
     def maybeOptimize(self):
         # have two weeks passed?
@@ -693,13 +719,6 @@ title="%s">%s</button>''' % (
         deck = self._selectedDeck()
         if not deck:
             return
-        if deck['dyn']:
-            showWarning(_("""\
-As cards are removed from a filtered deck as they are answered, viewing the \
-statistics of a filtered deck will only show you reviews for cards with \
-multiple steps. To get an accurate report, please empty the filtered deck \
-and check the statistics for a home deck instead."""))
-            return
         aqt.stats.DeckStats(self)
 
     def onPrefs(self):
@@ -723,6 +742,13 @@ and check the statistics for a home deck instead."""))
     # Importing & exporting
     ##########################################################################
 
+    def handleImport(self, path):
+        import aqt.importing
+        if not os.path.exists(path):
+            return showInfo(_("Please use File>Import to import this file."))
+
+        aqt.importing.importFile(self, path)
+
     def onImport(self):
         import aqt.importing
         aqt.importing.onImport(self)
@@ -798,16 +824,24 @@ and check the statistics for a home deck instead."""))
     def newMsg(self, data):
         aqt.update.showMessages(self, data)
 
-    def clockIsOff(self):
-        showWarning("""\
+    def clockIsOff(self, diff):
+        diffText = ngettext("%s second", "%s seconds", diff)
+        warn = _("""\
 In order to ensure your collection works correctly when moved between \
-devices, Anki requires the system clock to be set correctly. Your system \
-clock appears to be wrong by more than 5 minutes.
+devices, Anki requires your computer's internal clock to be set correctly. \
+The internal clock can be wrong even if your system is showing the correct \
+local time.
 
-This can be because the \
-clock is slow or fast, because the date is set incorrectly, or because \
-the timezone or daylight savings information is incorrect. Please correct \
-the problem and restart Anki.""")
+Please go to the time settings on your computer and check the following:
+
+- AM/PM
+- Clock drift
+- Day, month and year
+- Timezone
+- Daylight savings
+
+Difference to correct time: %s.""") % diffText
+        showWarning(warn)
         self.app.closeAllWindows()
 
     # Count refreshing
@@ -879,11 +913,16 @@ will be lost. Continue?"""))
 
     def onCheckMediaDB(self):
         self.progress.start(immediate=True)
-        (nohave, unused) = self.col.media.check()
+        (nohave, unused, invalid) = self.col.media.check()
         self.progress.finish()
         # generate report
         report = ""
+        if invalid:
+            report += _("Invalid encoding; please rename:")
+            report += "\n" + "\n".join(invalid)
         if unused:
+            if report:
+                report += "\n\n\n"
             report += _(
                 "In media folder but not used by any cards:")
             report += "\n" + "\n".join(unused)
@@ -894,7 +933,8 @@ will be lost. Continue?"""))
                 "Used on cards but missing from media folder:")
             report += "\n" + "\n".join(nohave)
         if not report:
-            report = _("No unused or missing files found.")
+            tooltip(_("No unused or missing files found."))
+            return
         # show report and offer to delete
         diag = QDialog(self)
         diag.setWindowTitle("Anki")
@@ -1050,6 +1090,8 @@ will be lost. Continue?"""))
         elif isWin:
             # make sure ctypes is bundled
             from ctypes import windll, wintypes
+            _dummy = windll
+            _dummy = wintypes
 
     def maybeHideAccelerators(self, tgt=None):
         if not self.hideMenuAccels:
@@ -1106,7 +1148,5 @@ Please ensure a profile is open and Anki is not busy, then try again."""),
         # import
         if not isinstance(buf, unicode):
             buf = unicode(buf, "utf8", "ignore")
-        if not os.path.exists(buf):
-            return showInfo(_("Please use File>Import to import this file."))
-        import aqt.importing
-        aqt.importing.importFile(self, buf)
+
+        self.handleImport(buf)
