@@ -2,8 +2,12 @@
 # License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html
 
 from __future__ import division
+import socket
+import time
+import traceback
+import gc
+
 from aqt.qt import *
-import   socket, time, traceback, gc
 import aqt
 from anki import Collection
 from anki.sync import Syncer, RemoteServer, FullSyncer, MediaSyncer, \
@@ -11,6 +15,7 @@ from anki.sync import Syncer, RemoteServer, FullSyncer, MediaSyncer, \
 from anki.hooks import addHook, remHook
 from aqt.utils import tooltip, askUserDialog, showWarning, showText, showInfo
 
+
 # Sync manager
 ######################################################################
 
@@ -26,6 +31,7 @@ class SyncManager(QObject):
             auth = self._getUserPass()
             if not auth:
                 return
+            self.pm.profile['syncUser'] = auth[0]
             self._sync(auth)
         else:
             self._sync()
@@ -50,6 +56,10 @@ class SyncManager(QObject):
             self.mw.app.processEvents()
             self.thread.wait(100)
         self.mw.progress.finish()
+        if self.thread.syncMsg:
+            showText(self.thread.syncMsg)
+        if self.thread.uname:
+            self.pm.profile['syncUser'] = self.thread.uname
         def delayedInfo():
             if self._didFullUp and not self._didError:
                 showInfo(_("""\
@@ -116,8 +126,8 @@ Please visit AnkiWeb, upgrade your deck, then try again."""))
             self._checkFailed()
         elif evt == "mediaSanity":
             showWarning(_("""\
-A problem occurred while syncing media. Please sync again and Anki will \
-correct the issue."""))
+A problem occurred while syncing media. Please use Tools>Check Media, then \
+sync again to correct the issue."""))
         elif evt == "noChanges":
             pass
         elif evt == "fullSync":
@@ -139,24 +149,24 @@ and try again.""")
             return _("""\
 The connection to AnkiWeb timed out. Please check your network \
 connection and try again.""")
-        elif "500" in err:
+        elif "code: 500" in err:
             return _("""\
 AnkiWeb encountered an error. Please try again in a few minutes, and if \
 the problem persists, please file a bug report.""")
-        elif "501" in err:
+        elif "code: 501" in err:
             return _("""\
 Please upgrade to the latest version of Anki.""")
         # 502 is technically due to the server restarting, but we reuse the
         # error message
-        elif "502" in err:
+        elif "code: 502" in err:
             return _("AnkiWeb is under maintenance. Please try again in a few minutes.")
-        elif "503" in err:
+        elif "code: 503" in err:
             return _("""\
 AnkiWeb is too busy at the moment. Please try again in a few minutes.""")
-        elif "504" in err:
+        elif "code: 504" in err:
             return _("504 gateway timeout error received. Please try temporarily disabling your antivirus.")
-        elif "409" in err:
-            return _("A previous sync failed; please try again in a few minutes.")
+        elif "code: 409" in err:
+            return _("Only one client can access AnkiWeb at a time. If a previous sync failed, please try again in a few minutes.")
         elif "10061" in err or "10013" in err:
             return _(
                 "Antivirus or firewall software is preventing Anki from connecting to the internet.")
@@ -164,8 +174,10 @@ AnkiWeb is too busy at the moment. Please try again in a few minutes.""")
             return _(
                 "Server not found. Either your connection is down, or antivirus/firewall "
                 "software is blocking Anki from connecting to the internet.")
-        elif "407" in err:
+        elif "code: 407" in err:
             return _("Proxy authentication required.")
+        elif "code: 413" in err:
+            return _("Your collection or a media file is too large to sync.")
         return err
 
     def _getUserPass(self):
@@ -263,8 +275,12 @@ class SyncThread(QThread):
         self.media = media
 
     def run(self):
+        # init this first so an early crash doesn't cause an error
+        # in the main thread
+        self.syncMsg = ""
+        self.uname = ""
         try:
-            self.col = Collection(self.path)
+            self.col = Collection(self.path, log=True)
         except:
             self.fireEvent("corrupt")
             return
@@ -321,12 +337,9 @@ class SyncThread(QThread):
             ret = self.client.sync()
         except Exception, e:
             log = traceback.format_exc()
-            try:
-                err = unicode(e[0], "utf8", "ignore")
-            except:
-                # number, exception with no args, etc
-                err = ""
-            if "Unable to find the server" in err:
+            err = repr(str(e))
+            if ("Unable to find the server" in err or
+                "Errno 2" in err):
                 self.fireEvent("offline")
             else:
                 if not err:
@@ -349,8 +362,14 @@ class SyncThread(QThread):
         # save and note success state
         if ret == "noChanges":
             self.fireEvent("noChanges")
-        else:
+        elif ret == "success":
             self.fireEvent("success")
+        elif ret == "serverAbort":
+            self.fireEvent("error", self.client.syncMsg)
+        else:
+            self.fireEvent("error", "Unknown sync return code.")
+        self.syncMsg = self.client.syncMsg
+        self.uname = self.client.uname
         # then move on to media sync
         self._syncMedia()
 
@@ -399,7 +418,7 @@ class SyncThread(QThread):
 ######################################################################
 
 CHUNK_SIZE = 65536
-import httplib, httplib2, errno
+import httplib, httplib2
 from cStringIO import StringIO
 from anki.hooks import runHook
 
@@ -413,6 +432,8 @@ def _incrementalSend(self, data):
             raise httplib.NotConnected()
     # if it's not a file object, make it one
     if not hasattr(data, 'read'):
+        if isinstance(data, unicode):
+            data = data.encode("utf8")
         data = StringIO(data)
     while 1:
         block = data.read(CHUNK_SIZE)
@@ -424,71 +445,49 @@ def _incrementalSend(self, data):
 httplib.HTTPConnection.send = _incrementalSend
 
 # receiving in httplib2
+# this is an augmented version of httplib's request routine that:
+# - doesn't assume requests will be tried more than once
+# - calls a hook for each chunk of data so we can update the gui
 def _conn_request(self, conn, request_uri, method, body, headers):
-    for i in range(2):
-        try:
-            if conn.sock is None:
-              conn.connect()
-            conn.request(method, request_uri, body, headers)
-        except socket.timeout:
-            raise
-        except socket.gaierror:
-            conn.close()
-            raise httplib2.ServerNotFoundError(
-                "Unable to find the server at %s" % conn.host)
-        except httplib2.ssl_SSLError:
-            conn.close()
-            raise
-        except socket.error, e:
-            err = 0
-            if hasattr(e, 'args'):
-                err = getattr(e, 'args')[0]
-            else:
-                err = e.errno
-            if err == errno.ECONNREFUSED: # Connection refused
-                raise
-        except httplib.HTTPException:
-            # Just because the server closed the connection doesn't apparently mean
-            # that the server didn't send a response.
-            if conn.sock is None:
-                if i == 0:
-                    conn.close()
-                    conn.connect()
-                    continue
-                else:
-                    conn.close()
-                    raise
-            if i == 0:
-                conn.close()
-                conn.connect()
-                continue
-            pass
-        try:
-            response = conn.getresponse()
-        except (socket.error, httplib.HTTPException):
-            if i == 0:
-                conn.close()
-                conn.connect()
-                continue
-            else:
-                raise
+    try:
+        if conn.sock is None:
+          conn.connect()
+        conn.request(method, request_uri, body, headers)
+    except socket.timeout:
+        raise
+    except socket.gaierror:
+        conn.close()
+        raise httplib2.ServerNotFoundError(
+            "Unable to find the server at %s" % conn.host)
+    except httplib2.ssl_SSLError:
+        conn.close()
+        raise
+    except socket.error, e:
+        conn.close()
+        raise
+    except httplib.HTTPException:
+        conn.close()
+        raise
+    try:
+        response = conn.getresponse()
+    except (socket.error, httplib.HTTPException):
+        raise
+    else:
+        content = ""
+        if method == "HEAD":
+            response.close()
         else:
-            content = ""
-            if method == "HEAD":
-                response.close()
-            else:
-                buf = StringIO()
-                while 1:
-                    data = response.read(CHUNK_SIZE)
-                    if not data:
-                        break
-                    buf.write(data)
-                    runHook("httpRecv", len(data))
-                content = buf.getvalue()
-            response = httplib2.Response(response)
-            if method != "HEAD":
-                content = httplib2._decompressContent(response, content)
-        break
+            buf = StringIO()
+            while 1:
+                data = response.read(CHUNK_SIZE)
+                if not data:
+                    break
+                buf.write(data)
+                runHook("httpRecv", len(data))
+            content = buf.getvalue()
+        response = httplib2.Response(response)
+        if method != "HEAD":
+            content = httplib2._decompressContent(response, content)
     return (response, content)
 
 httplib2.Http._conn_request = _conn_request
